#include <WiFi.h>
#include <ESP32Servo.h>

// Replace with your network credentials
const char* ssid = "EngineeringStudent";
const char* password = "cls2024!";

// Pin assignments
const int servoPin = 7;   // GPIO pin for the servo

Servo myServo;
WiFiServer server(80);

void setup() {
  Serial.begin(115200);
  myServo.attach(servoPin);  // Attach the servo to GPIO 21

  // Connect to WiFi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }
  Serial.println("Connected to WiFi");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Start the server
  server.begin();
}

void loop() {
  WiFiClient client = server.available();  // Check if a client has connected
  if (client) {
    String request = client.readStringUntil('\r');  // Read the incoming request
    client.flush();

    // Check if we received the move command with the position
    if (request.indexOf("/move?pos=") != -1) {
      int posStart = request.indexOf("pos=") + 4;
      int posEnd = request.indexOf(" ", posStart);
      if (posEnd == -1) posEnd = request.length();
      String posString = request.substring(posStart, posEnd);
      int pos = posString.toInt();
      if (pos >= 0 && pos <= 180) {
        myServo.write(pos);  // Move servo to the calculated position
      }
    }

    // Send HTML page
    client.println("HTTP/1.1 200 OK");
    client.println("Content-Type: text/html");
    client.println("Connection: close");
    client.println();
    client.println("<html><body>");
    client.println("<h1>Control Servo with Joystick</h1>");
    client.println("<div id='joystick' style='width:200px;height:200px;border:1px solid black;position:relative;'>");
    client.println("<div id='knob' style='width:50px;height:50px;background-color:red;position:absolute;border-radius:50%;top:75px;left:75px;'></div>");
    client.println("</div>");
    client.println("<script>");
    client.println("var joystick = document.getElementById('joystick');");
    client.println("var knob = document.getElementById('knob');");
    client.println("var startX, startY, angle;");
    client.println("joystick.addEventListener('mousedown', function(e) {");
    client.println("  startX = e.clientX - joystick.offsetLeft - 25;");
    client.println("  startY = e.clientY - joystick.offsetTop - 25;");
    client.println("  document.onmousemove = function(e) {");
    client.println("    var x = e.clientX - joystick.offsetLeft - 25;");
    client.println("    var y = e.clientY - joystick.offsetTop - 25;");
    client.println("    var distance = Math.sqrt(x*x + y*y);");
    client.println("    if (distance > 75) {");
    client.println("      x = (x / distance) * 75;");
    client.println("      y = (y / distance) * 75;");
    client.println("    }");
    client.println("    knob.style.left = 75 + x + 'px';");
    client.println("    knob.style.top = 75 + y + 'px';");
    client.println("    angle = Math.atan2(y, x) * (180 / Math.PI) + 90;");
    client.println("    fetch('/move?pos=' + Math.round(angle));");
    client.println("  };");
    client.println("  document.onmouseup = function() { document.onmousemove = null; };");
    client.println("});");
    client.println("</script>");
    client.println("</body></html>");
    delay(1);
  }
}


